#include "StdAfx.h"
#include "UpdaterDocument.h"
#include "XMLUtil.h"
#include <atlpath.h>
#include "HttpClient/httpclient.h"
#include "Binding.h"
#include "ScriptletContainerFactory.h"
#include "Application.h"
#include "CustomProgressBar.h"
#include "../../../common/ConfigXmlParser.h"
#include "UpdaterNotifyWindowQueue.h"
#include "LocalStorage/LocalStorage.h"
#include "CheckServiceUpgradeHelper.h"
#include "../../../common/drive.h"
#include <memory>
#include "TrayIconHandler.h"

namespace tipupgrader
{
const TCHAR CONFIG_XML_FILENAME[]				= _T("NToolsUpdater.xml");
const TCHAR CONFIG_CONTEXT_NAME[]				= _T("config");
const TCHAR CONFIG_FILE_NAME[]					= _T("NToolsUpdaterConfig");
const TCHAR CHECK_EVENT_TIME_CONTEXT_NAME[]		= _T("checkeventtime");

UpdaterDocument::UpdaterDocument():m_ConfigModel(this),m_CommonModel(this)
{
	m_bUpdateTab = TRUE;
	m_PendingService = NULL;
	m_bIsShowConfig = FALSE;
	m_bShowTooltipCloseMainViewFirstTime = FALSE;

// 	TCHAR statUrl[tipbase::MAXURL] = {0, };
// 	tipcommon::ConfigXmlParser::GetURL(GetConfigXmlName(), tipcommon::Element::StatUrl, statUrl, tipbase::MAXURL);
	m_NotifyWindowQueue = new UpdaterNotifyWindowQueue;
	m_bIsShowingNotifyWindow = FALSE;
	m_bInitListLayout = FALSE;
	InitializeCriticalSection(&m_criticalSection);
}

UpdaterDocument::~UpdaterDocument()
{	
	if (m_NotifyWindowQueue)
		delete m_NotifyWindowQueue;
	m_NotifyWindowQueue = NULL;
	DeleteCriticalSection(&m_criticalSection);
}

void UpdaterDocument::SetPendingService(TipServiceData* service) 
{ 
	m_PendingService = service; 
	if( NULL != service )
		TrayIconHandler::CreateInstacne()->SetUpdatingToolTip();
	else
		TrayIconHandler::CreateInstacne()->SetEmptyToolTip();
}

BOOL UpdaterDocument::Open()
{
	BOOL bSuccess = __super::Open();
	if (bSuccess)
	{
		LoadConfigModel();
		LoadCheckEventTime();
		CComVariant var = this->GetProperty(_T("config.autostart"));
		CComVariant var1 = this->GetProperty(_T("config.checkcorevalue"));

		this->AddDataContext(_T("updateinfo"));
		this->AddDataContext(_T("installinfo"));
		ShowUpdateTab(TRUE);
		SetListSelect(0, 0);
	}
	return bSuccess;
}
BOOL UpdaterDocument::CheckInstallInfo()
{
	TipServiceData* service = NULL;

	AgentDLLWrapper* agentDllWrapper = NULL;
	NAVERAPP_INSTALL_STATUS InstallStatus;
	int numOfNotInstalled = 0;
	int numOfNotUpdated = 0;
	BOOL bChanged = FALSE;
	InstallInfo::type oldInfo = InstallInfo::NOTINSTALLED;
	
	int count = GetServiceCount();
	for (int i = 0; i < count; i++)
	{
		service = GetService(i);
		oldInfo = service->GetInstallInfo();

		agentDllWrapper = service->GetAgentDll();
		if (agentDllWrapper)
		{
			agentDllWrapper->GetInstalledStatus(InstallStatus);
			if (oldInfo != InstallStatus)
				bChanged = TRUE;

			switch(InstallStatus)
			{
			case NotInstalled:
				numOfNotInstalled++;
				if (oldInfo != InstallInfo::NOTINSTALLED)
					service->SetInstallInfo(InstallInfo::NOTINSTALLED);
				break;
			case OldVersion:
				numOfNotUpdated++;
				if (oldInfo != InstallInfo::OLDVERSION)
					service->SetInstallInfo(InstallInfo::OLDVERSION);
				
				break;
			case LatestVersion:
				//numOfNotUpdated++;
				if (oldInfo != InstallInfo::RECENTVERSION)
					service->SetInstallInfo(InstallInfo::RECENTVERSION);
				break;
			}
			agentDllWrapper = NULL;
		}
		else
		{
			numOfNotInstalled++;
			if (oldInfo != InstallInfo::NOTINSTALLED)
				service->SetInstallInfo(InstallInfo::NOTINSTALLED);
		}
	}

	if (bChanged)
	{
		int numOfInstalled = GetServiceCount() - numOfNotInstalled;
		SetInstalledListCount(numOfInstalled);
		SetUpdateListCount(numOfNotUpdated);
		SetInstallListCount(numOfNotInstalled);
		return TRUE;
	}
	return FALSE;
}

BOOL UpdaterDocument::UpdateInstallInfo(LPCTSTR guid)
{
	int numOfNotInstalled = GetInstallListCount();
	int numOfNotUpdated = GetUpdateListCount();
	
	BOOL bChangedInfo = FALSE;

	TipServiceData* serivceData = SearchService(guid);
	if (serivceData)
	{
		InstallInfo::type oldInfo = serivceData->GetInstallInfo();

		AgentDLLWrapper* agentDllWrapper = NULL;
		agentDllWrapper = serivceData->GetAgentDll();

		NAVERAPP_INSTALL_STATUS InstallStatus;
		agentDllWrapper->GetInstalledStatus(InstallStatus);

		InstallInfo::type newInfo;
		switch(InstallStatus)
		{
		case NotInstalled:
			newInfo = InstallInfo::NOTINSTALLED;
			break;
		case OldVersion:
			newInfo = InstallInfo::OLDVERSION;
			break;
		case LatestVersion:
			newInfo = InstallInfo::RECENTVERSION;
			break;
		}
		if (newInfo == oldInfo)
			return bChangedInfo;

		if ((oldInfo == InstallInfo::OLDVERSION || oldInfo == InstallInfo::RECENTVERSION)
			&& newInfo == InstallInfo::NOTINSTALLED)
		{
			bChangedInfo = TRUE;
			numOfNotInstalled ++;
			//   ̾ٸ Ʈ Ʈ 
			if (oldInfo == InstallInfo::OLDVERSION)
				numOfNotUpdated--;
		}
		else if ((newInfo == InstallInfo::OLDVERSION || newInfo == InstallInfo::RECENTVERSION)
			&& (oldInfo == InstallInfo::NOTINSTALLED))
		{
			bChangedInfo = TRUE;
			numOfNotInstalled --;
			//   ̶ Ʈ Ʈ 
			if (newInfo == InstallInfo::OLDVERSION)
				numOfNotUpdated ++;
		}
		else if (( newInfo == InstallInfo::RECENTVERSION)
			&& (oldInfo == InstallInfo::OLDVERSION))
		{
			bChangedInfo = TRUE;
			//   ̾ٸ Ʈ Ʈ 
			if (oldInfo == InstallInfo::OLDVERSION)
				numOfNotUpdated --;
		}
		serivceData->SetInstallInfo(newInfo);

		int numOfInstalled = GetServiceCount() - numOfNotInstalled;
		SetInstalledListCount(numOfInstalled);
		SetUpdateListCount(numOfNotUpdated);
		SetInstallListCount(numOfNotInstalled);
		return bChangedInfo;
	}

	return bChangedInfo;
}
CTrayIcon* UpdaterDocument::GetTrayIcon()
{
	return TrayIconHandler::CreateInstacne()->GetTrayIcon();
}
BOOL UpdaterDocument::Reload()
{
	return CheckInstallInfo();
}

//////////////////////////////////////////////////////////////////////////
//UpdaterConfigModel
//////////////////////////////////////////////////////////////////////////
UpdaterConfigModel::UpdaterConfigModel(TipDocument* doc):m_Document(doc)
{
}

class ConfigLoader
{
public:
	void SetDefaultOption(UpdaterConfigModel *config, tipcommon::TipDocument *document )
	{		
		if( NULL == config || NULL == document )
			return;

		document->AddDataContext(_T("config"));
		config->SetUseNotifyUpdate	(TRUE);
		config->SetUseTrayIcon		(TRUE);
		config->SetUseCheckCoreValue(TRUE);
	}

	BOOL Load(UpdaterConfigModel *config, IDataContext *dataContext)
	{
		ATLASSERT( config );
		if( NULL == config)
			return FALSE;

		TCHAR config_file_position[1024];
		if( FALSE == Drive::GetCurrentUserLocalStorageFolderFileName( config_file_position, 1024, CONFIG_FILE_NAME) )
		{
			return FALSE;
		}

		std::auto_ptr<tipbase::LocalStorageManager> storage(new tipbase::LocalStorageManager(new tipbase::FileTool((TCHAR *)config_file_position)));

		int datasize = storage->Load( CONFIG_CONTEXT_NAME, NULL, 0, 0 );
		if( 0 == datasize )
		{
			return FALSE;
		}

		char *data = new char[datasize+2];
		if( NULL == data )
		{
			return FALSE;
		}

		storage->Load( CONFIG_CONTEXT_NAME, NULL, data, datasize );
		data[datasize] = 0;

		TiXmlDocument doc;
		doc.Parse(data);

		TiXmlElement* element =  doc.FirstChildElement();
		XMLDataProvider xmlProvider( dataContext );
		xmlProvider.Load(element);

		// [:2008-12-18] Memory Leak  : delete 
		delete [] data;

		return TRUE;
	}
};

class ConfigSaver
{
public:
	void MakeXML(CString &strXML, CString elementXML)
	{
		strXML += _T("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\r\n");
		strXML += _T("<root>\r\n");
		strXML += elementXML;
		strXML += _T("</root>");
	}

	CString BoolToString(BOOL value)
	{
		if( value )
			return _T("true");

		return _T("false");
	}

	CString MakeConfigXML(	
		BOOL bNotifyUpdate, 
		BOOL bShowTray, 
		BOOL bCheckCoreValue )
	{
		CString strElement;
		strElement = _T("<config>");
		strElement += _T("<notifyupdate>");
		strElement += BoolToString(bNotifyUpdate);
		strElement += _T("</notifyupdate>");
		strElement += _T("<usetrayicon>");
		strElement += BoolToString(bShowTray);
		strElement += _T("</usetrayicon>");
		strElement += _T("<checkcorevalue>");
		strElement += BoolToString(bCheckCoreValue);
		strElement += _T("</checkcorevalue>");
		strElement += _T("</config>");

		CString strXML;
		MakeXML(strXML, strElement);
		USES_CONVERSION;

		return strXML;
	}

	BOOL Save( UpdaterConfigModel *config )
	{
		ATLASSERT( config );
		if( !config )
			return FALSE;

		TCHAR config_file_position[1024];
		Drive::GetCurrentUserLocalStorageFolderFileName( config_file_position, 1024, CONFIG_FILE_NAME);
		std::auto_ptr<tipbase::LocalStorageManager> storage(new tipbase::LocalStorageManager(new tipbase::FileTool((TCHAR *)config_file_position)));

		BOOL bNotifyUpdate, bShowTray,bCheckCoreValue;
		GetConfigData( config, bNotifyUpdate, bShowTray, bCheckCoreValue );

		CString xml = MakeConfigXML( bNotifyUpdate, bShowTray, bCheckCoreValue );
		CT2CA xmlAscii(xml);

		if( 0 == storage->Save( CONFIG_CONTEXT_NAME, NULL, (const char *)xmlAscii, strlen(xmlAscii) ) )
			return FALSE;

		return TRUE;
	}

	void GetConfigData( UpdaterConfigModel *config, 
		BOOL& bNotifyUpdate, 
		BOOL& bShowTray, 
		BOOL& bCheckCoreValue )
	{
		bNotifyUpdate	= config->IsUseNotifyUpdate();
		bShowTray		= config->IsUseTrayIcon();
		bCheckCoreValue	= config->IsUseCheckCoreValue();
	}
};

class CheckEventTimeLoader
{
public:
	void ParseXML( const char *data, UpdaterDocument *doc )
	{
		TiXmlDocument xmlDoc;
		xmlDoc.Parse(data);
		
		TiXmlElement* element =  xmlDoc.FirstChildElement();

		SimpleMapDataContext dataContext;
		XMLDataProvider xmlData(&dataContext);
		xmlData.Load( element );

		IDataContext* installlistDataContext = dataContext.GetChild(_T("servicelist"));
		if (installlistDataContext != NULL)
		{
			int count = installlistDataContext->GetCount();
			for(int index = 0; index < count; index++)
			{
				IDataContext* serviceDataContext = installlistDataContext->GetChild(index);
				if (serviceDataContext != NULL)
				{
					CComVariant var = serviceDataContext->GetProperty(_T("guid"));
					CString guid = var;
					var = serviceDataContext->GetProperty(_T("chekc_event_time"));
					CString eventtime = var;
					
					if( eventtime == _T("") )
						continue;

					TipServiceData *service = doc->SearchService(guid);
					if( NULL == service )
						continue;

					CheckServiceUpgradeHelper helper(service);
					helper.SetServiceCheckTime(eventtime);
				}			
			}
		}	
		
	}

	void Load( UpdaterDocument *doc )
	{
		ATLASSERT( doc );
		if( NULL == doc)
			return;


		TCHAR config_file_position[1024];
		Drive::GetCurrentUserLocalStorageFolderFileName( config_file_position, 1024, CONFIG_FILE_NAME);
		std::auto_ptr<tipbase::LocalStorageManager> storage(new tipbase::LocalStorageManager(new tipbase::FileTool((TCHAR *)config_file_position)));

		int datasize = storage->Load( CHECK_EVENT_TIME_CONTEXT_NAME, NULL, 0, 0 );
		if( 0 == datasize )
		{
			return;
		}

		char *data = new char[datasize+2];
		if( NULL == data )
			return;

		storage->Load( CHECK_EVENT_TIME_CONTEXT_NAME, NULL, data, datasize );
		data[datasize] = 0;

		ParseXML(data, doc);

		// [:2008-12-18] Memory Leak  : delete 
		delete [] data;
	}
};

class CheckEventTimeSaver
{
public:
	void MakeXML(CString &strXML, CString elementXML)
	{
		strXML += _T("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>\r\n");
		strXML += _T("<root>\r\n");
		strXML += elementXML;
		strXML += _T("</root>");
	}

	CString MakeCheckEventTimeXML( UpdaterDocument *doc )
	{
		CString strElement = _T("");
		int count = doc->GetServiceCount();
		TipServiceData *service = NULL;
		for (int i = 0; i < count; i++)
		{
			service = doc->GetService(i);
			if( NULL == service )
				continue;
		
			CheckServiceUpgradeHelper helper(service);
			
			CString xml;
			CString id = helper.GetGuid();
			CString checkEventTime = helper.GetServiceCheckTime();
			if( checkEventTime == _T("") )
				continue;

			xml.Format( _T("<service guid=\"%s\"><chekc_event_time>%s</chekc_event_time></service>\r\n"), id, checkEventTime );
			strElement += xml;
		}

		CString strXML;
		MakeXML(strXML, strElement);
		USES_CONVERSION;

		return strXML;
	}

	BOOL Save( UpdaterDocument *doc )
	{
		TCHAR config_file_position[1024];
		Drive::GetCurrentUserLocalStorageFolderFileName( config_file_position, 1024, CONFIG_FILE_NAME);
		std::auto_ptr<tipbase::LocalStorageManager> storage(new tipbase::LocalStorageManager(new tipbase::FileTool((TCHAR *)config_file_position)));
		
		CString xml = MakeCheckEventTimeXML( doc );
		CT2CA xmlAscii(xml);

		if( 0 == storage->Save( CHECK_EVENT_TIME_CONTEXT_NAME, NULL, (const char *)xmlAscii, strlen(xmlAscii) ) )
			return FALSE;

		return TRUE;
	}
};

void UpdaterConfigModel::Init()
{
	ATLASSERT(m_Document);
	ConfigLoader loader;
	if( FALSE == loader.Load(this, m_Document->GetRoot()) )
	{
		loader.SetDefaultOption( this, m_Document );
	}
}

BOOL UpdaterConfigModel::Save()
{
	ConfigSaver saver;
	return saver.Save(this);
}

BOOL UpdaterDocument::SaveConfigModel()
{
	return m_ConfigModel.Save();
}

void UpdaterDocument::LoadConfigModel()
{
//	AddDataContext(_T("config"));
	m_ConfigModel.Init();
}

void UpdaterDocument::LoadCheckEventTime()
{
	CheckEventTimeLoader loader;
	loader.Load(this);
}

BOOL UpdaterDocument::SaveCheckEventTime()
{
	CheckEventTimeSaver saver;
	return saver.Save(this);
}

UpdaterXMLValidater::UpdaterXMLValidater()
{
	m_TypeDefiner.SetAt(_T("startpage.url"), UrlType);
	m_TypeDefiner.SetAt(_T("defaultsearch.clsid"), GuidType);
	m_TypeDefiner.SetAt(_T("defaultsearch.url"), UrlType);
	m_TypeDefiner.SetAt(_T("startpage.changed.copy"), StringType);
	m_TypeDefiner.SetAt(_T("defaultsearch.changed.copy"), StringType);
	m_TypeDefiner.SetAt(_T("guid"), GuidType);
	m_TypeDefiner.SetAt(_T("name"), StringType);
	m_TypeDefiner.SetAt(_T("introtext"), StringType);
	m_TypeDefiner.SetAt(_T("icon.url"), UrlType);
	m_TypeDefiner.SetAt(_T("alarmicon.url"), UrlType);
	m_TypeDefiner.SetAt(_T("exposureweight"), NumberType);
	m_TypeDefiner.SetAt(_T("introduce.url"), UrlType);
	m_TypeDefiner.SetAt(_T("setupfile.url"), UrlType);
	m_TypeDefiner.SetAt(_T("agentdllxml.url"), UrlType);
	m_TypeDefiner.SetAt(_T("update.silent"), BooleanType);
	m_TypeDefiner.SetAt(_T("update.copy"), StringType);
	m_TypeDefiner.SetAt(_T("update.showinfo"), BooleanType);
	m_TypeDefiner.SetAt(_T("update.infourl"), UrlType);
}
BOOL UpdaterXMLValidater::IsValid(LPCTSTR key, LPCTSTR data)
{
	BOOL bRetVal = TRUE;
	CString dataKey(key);
	CComVariant var;
	CAtlMap<CString, valueType>::CPair* pair = m_TypeDefiner.Lookup(dataKey);
	if (pair != NULL)
	{
		var = pair->m_value;
		switch(var.intVal)
		{
		case UrlType:
			bRetVal = IsValidURL(data);
			break;
		case GuidType:
			bRetVal = IsValidGUID(data);
			break;
		case NumberType:
			bRetVal = IsValidNumber(data);
			break;
		case BooleanType:
			bRetVal = IsValidBoolean(data);
			break;
		case StringType:
			bRetVal = IsValidString(data);
			break;
		}
	}

	return bRetVal;
}

};
